require 'test_helper'
require_relative "test_page.rb"
require_relative "test_refreshable.rb"

describe "Datarobot::AiApi::AI" do
  before do
    @key = "testingKey"
  end

  it "is sane" do
    Datarobot::AiApi::AI.new
    # if we make it to here, nothing has exploded
    assert true
  end

  describe "when authorized" do
    before do
      Datarobot::AiApi.configure!(api_key: @key)
      @klass = Datarobot::AiApi::AI
      @id = '5d322e625c142b15dff826fc'
    end

    it_behaves_like "a paginated resource"
    it_behaves_like "a refreshable resource"

    it "can list AIs", vcr: true do
      ais = Datarobot::AiApi::AI.all
      assert ais.is_a? Datarobot::AiApi::Page
      assert_equal ais.length, 15
      assert_includes(ais.map(&:name), "Bird classifier")
    end

    it "can find AIs", vcr: true do
      ai = Datarobot::AiApi::AI.find('5d322e625c142b15dff826fc')
      assert ai.is_a? Datarobot::AiApi::AI
      assert_equal ai.name, "Bird classifier"
    end

    it "handles when the AI ID is nil" do
      assert_raises Datarobot::AiApi::NotFoundError do
        ai = Datarobot::AiApi::AI.find(nil)
      end
    end

    it "can create AIs", vcr: true do
      ai = Datarobot::AiApi::AI.create(name: 'a testing AI')
      assert ai.is_a? Datarobot::AiApi::AI
      assert_equal ai.name, "a testing AI"
    end

    it "can delete AIs", vcr: true do
      ai = Datarobot::AiApi::AI.delete('5d5c72af42eabf0c8b66f2eb')
      assert_nil ai
    end

    it "can get AI outputs", vcr: true do
      ai = Datarobot::AiApi::AI.find('5d322e625c142b15dff826fc')
      outputs = ai.outputs
      assert outputs.is_a? Datarobot::AiApi::Page
      assert_includes outputs.map(&:name), "class"
    end

    it "can get AI datasets", vcr: true do
      ai = Datarobot::AiApi::AI.find('5d322e625c142b15dff826fc')
      datasets = ai.datasets
      assert datasets.length == ai.dataset_count
      assert datasets.is_a? Datarobot::AiApi::Page
      assert_includes datasets.map(&:name), "image_birds_multiclass_6x100.csv"
    end

    it "can add datasets to AIs", vcr: true do
      ai = Datarobot::AiApi::AI.find('5d322e625c142b15dff826fc')
      was_added = ai.add_dataset('5d5d914e42eabf0f1166f29f')
      assert was_added
    end

    it "can add learning sessions to AIs", vcr: true do
      ai = Datarobot::AiApi::AI.find('5d322e625c142b15dff826fc')
      was_added = ai.add_learning_session('5d41de29d8c81f0587732a4f')
      assert was_added
    end

    it "404s if there is no matching output", vcr: true do
      ai = Datarobot::AiApi::AI.find('5d4205d6d8c81f05fe732a42')
      data = {'date': '2019-08-26', 'league_id': '1845', 'league': 'German Bundesliga', 'team1': 'Werder Bremen', 'team2': 'Hamburg SV', 'spi1': '64.54', 'spi2': '43.91'}

      assert_raises Datarobot::AiApi::NotFoundError do
        predictions = ai.predict('not_a_valid_target', data)
      end
    end

    it "can predict things", vcr: true do
      ai = Datarobot::AiApi::AI.find('5d4205d6d8c81f05fe732a42')
      data = {'date': '2019-08-26', 'league_id': '1845', 'league': 'German Bundesliga', 'team1': 'Werder Bremen', 'team2': 'Hamburg SV', 'spi1': '64.54', 'spi2': '43.91'}
      predictions = ai.predict('score1', data)
      assert predictions.is_a? Datarobot::AiApi::Page

      prediction = predictions.first

      assert prediction.is_a? Datarobot::AiApi::Prediction
      assert_equal prediction.prediction, 3.7707404629 
    end

    it "can be trained", vcr: true do
      ai = Datarobot::AiApi::AI.create(name: 'test AI')
      ai.train('score1', './test/data/test.csv')
    end
  end

  describe "when not authorized" do
    before do
      Datarobot::AiApi.configure!(api_key: '')
    end

    it "can not list AIs", vcr: true do
      assert_raises Datarobot::AiApi::UnauthorizedError do 
        Datarobot::AiApi::AI.all
      end
    end

    it "can not find AIs", vcr: true do
      assert_raises Datarobot::AiApi::UnauthorizedError do 
        Datarobot::AiApi::AI.find('5d322e625c142b15dff826fc')
      end
    end

    it "can not create AIs", vcr: true do
      assert_raises Datarobot::AiApi::UnauthorizedError do 
        Datarobot::AiApi::AI.create(name: 'a testing AI')
      end
    end
  end
end